program DC7TU64;
{---------------------------------------------------------------------------}
{ Original Source from Peter Ochoisky 1992                                  }
{ Kopie fuer DD1US  de DC7TU                                                }
{ Programm zum Erzeugen der EPROMs fr den Rufzeichen-Geber von DL3PD/DL6LG }
{ Version fr EPROMs 2764 mit 8 kB Inhalt                                   }
{---------------------------------------------------------------------------}
{$R-,S-,I-,N-,L-,D-,v-}
uses crt,oiunit;

type
  zustand = (null , eins);

  const
    maxi   = $1fff;                         { EPROM-Typ 2764 = 8192 Bytes }
(*  maxi   = $7fff;                         { EPROM-Typ 27256=32767 Bytes }*)
(*
    cr     = #13;
    esc    = #27;
    dn     = #80;
    pgup   = #73;
    pgdn   = #81;
    home   = #71;
    ende   = #79;
*)
var
  stemp      : string;
  fname      : string[30];
  eprom      : array [0..maxi] of byte;
  input_file : file of byte;

  fuell_b  : byte;
  block    ,
  leer     ,
  antwort  ,
  ein      : char;

  z        ,
  x_anf    ,                          { X-Position Anfang Matrix auf Schirm }
  y_anf    ,                          { Y-Position Anfang Matrix auf Schirm }
  x_pos    ,                                        { mom. X-Pos auf Schirm }
  y_pos    ,                                        { dto. Y                }
  sc_byte  : integer;                           { Byte aus EPROM auf Schirm }

  i        ,                                                    { allgemein }
  bis      ,                                           { Automat-Endadresse }
  anfang   ,
  endadr   ,
  x_edit   ,
  x_m      : word;                                  { X-Koordinate im EPROM }

  automat     ,
  gespeichert ,
  geaendert   : boolean;
  long_dumm   : longint;


{---------------------------------------------------------------------------}
function sicher (mitteilung : string): boolean;
{---------------------------------------------------------------------------}
begin
  pieps_twit;
  sicher := false;
  gotoxy (8,5);
  clreol;
  normal;
  write (mitteilung , ' Sicher (J/N) ? ');
  inverse;

  repeat
    antwort := upcase (readkey);
  until antwort in ['J','N'];

  if antwort = 'J' then
    sicher := true;

  gotoxy (1,5);
  clreol;
end;                                                               { sicher }


{---------------------------------------------------------------------------}
procedure file_lesen;
{---------------------------------------------------------------------------}
begin
  gotoxy (1,5);
  write ('         Filename zum LESEN ? ');
  hol_str (normal , fname , 12 , inverse);

  gotoxy (1,5);
  clreol;

  if (fname = '')
  or not sicher ('FILE EINLESEN VON ' + fname + ' : ') then
    exit;

  write ('       <<   EPROM-Inhalt von "',fname,'" lesen...');

  x_m   := 0;
  abbruch := false;
{$i-}
  assign (input_file , fname);
  reset  (input_file);
  if ioresult > 0 then
    begin
      clreol;
      writeln ('> File nicht gefunden <'#7);
      exit;
    end;

  repeat
    read (input_file , eprom [x_m]);                 { EPROM von File lesen }
                                                   { und in Array schreiben }
    if x_m mod 256 = 0 then
      write (cr,'$',ihex(x_m));
    inc (x_m);                                              { nchstes Byte }
    tstop;
  until (x_m > maxi)
        or abbruch
        or eof (input_file);
  if ioresult > 0 then
    writeln ('Fehler READ');

  close (input_file);
  if ioresult > 0 then
    writeln ('Fehler CLOSE');
{$i+}

  dec (x_m);                                    { weil mit INC aufgehrt... }
  clreol;
  write (cr , '         $',ihex(x_m) ,' Bytes eingelesen...');

  x_m := 0;
  gespeichert := false;
  geaendert   := false;
end;                                                           { file_lesen }


{---------------------------------------------------------------------------}
procedure file_schreiben;
{---------------------------------------------------------------------------}
begin
  gotoxy (1,5);
  clreol;
  write ('         Filename zum SCHREIBEN ? ');
  hol_str (normal , fname , 12 , inverse);

  gotoxy (1,5);
  clreol;

  if (fname = '')
  or not sicher ('Schreiben auf ' + fname + ' : ') then
    exit;

  write ('       <<   EPROM-Inhalt auf "',fname,
         '" schreiben, Moment bitte...',cr);

  x_m   := 0;
  abbruch := false;

{$i-}
  assign (input_file , fname);
  rewrite  (input_file);
  if ioresult > 0 then
    writeln ('Fehler REWRITE');

  repeat
    write (input_file , eprom [x_m]);
    if x_m mod 256 = 0 then
      write (cr,'$',ihex(x_m));
    inc (x_m);                                              { nchstes Byte }
  until (x_m > maxi);
  if ioresult > 0 then
    writeln ('Fehler WRITE');

  close (input_file);
  if ioresult > 0 then
    writeln ('Fehler CLOSE');
{$i+}

  dec (x_m);                                    { weil mit INC aufgehrt... }
  clreol;
  write (cr , '         $',ihex(x_m) ,' Bytes geschrieben...');

  x_m := 0;
  gespeichert := true;
  geaendert   := false;
end;                                                       { file_schreiben }


{---------------------------------------------------------------------------}
procedure zeig_byte;                { Byte als senkrechte Punktreihe zeigen }
{---------------------------------------------------------------------------}
var
  i    : byte;
begin
  for i := 0 to 7 do                                   { 7 Punkte senkrecht }
    begin
      gotoxy (x_pos , y_anf + i);               { senkrecht runter zeichnen }
      if (sc_byte and (1 shl i)) = 0 then          { abtasten mit 1 shiften }
        write (leer)
      else
        write (block);
    end;
end;                                                            { zeig_byte }


{---------------------------------------------------------------------------}
procedure mach_bit (art : zustand);            { Wenn Matrix gendert wurde }
{---------------------------------------------------------------------------}
var
  neu_byte : byte;
begin
  geaendert := true;
  x_edit := x_m + round ((x_pos - x_anf) / 2);
                               { Adresse machen, welche editiert worden ist }
  if art = eins then                               { wenn Bit auf 1 gesetzt }
    neu_byte := eprom [x_edit] or (1 shl (y_pos - y_anf))
  else                                             { wenn Bit auf 0 gesetzt }
    neu_byte := eprom [x_edit] and (255 - (1 shl (y_pos - y_anf)));

  eprom [x_edit] := neu_byte;         { neues Byte in EPROM-Array schreiben }
  sc_byte := neu_byte;

  gotoxy (x_pos , 9);
  write (hex (neu_byte));                       { neues Byte als HEX zeigen }
end;                                                             { mach_bit }


{---------------------------------------------------------------------------}
procedure matrix_check;                { Matrix-Grenzen fuer Cursor checken }
{---------------------------------------------------------------------------}
begin
  if y_pos < y_anf then
    y_pos := y_anf;
  if y_pos > y_anf + 7 then
    y_pos := y_anf + 7;
  if x_pos < x_anf then
    x_pos := x_anf;
  if x_pos >= x_anf + 62 then
    x_pos := x_anf + 62;
end;                                                         { matrix_check }


{---------------------------------------------------------------------------}
procedure editier;                                      { Matrix bearbeiten }
{---------------------------------------------------------------------------}
begin
  x_pos := x_anf;
  y_pos := y_anf;

  repeat
    gotoxy (x_pos , y_pos);                           { an Cursor-Pos gehen }

    ein := upcase (readkey);

    if ein = '+' then
      begin
        mach_bit (eins);
        zeig_byte;
{       inc (x_pos,2);}
        matrix_check;
      end;


    if ein = '-' then
      begin
        mach_bit (null);
        zeig_byte;
{       inc (x_pos,2);}
        matrix_check;
      end;


    if ein = 'A' then
      begin                                          { Automatik-Vorfhrung }
        automat := true;
        bis     := $3ff;
        x_m     := 0;
        gotoxy (1,5);
        clreol;
        gotoxy (1,9);
        clreol;
        gotoxy (1,18);
        clreol;
        gotoxy (1,19);
        clreol;
        gotoxy (1,21);
        clreol;
        gotoxy (1,22);
        clreol;
      end;


    if (ein = 'F') then
      begin
        repeat
          gotoxy (x_anf,5);
          clreol;
          write ('FLLEN: Anfangsadresse Hi-Lo ($0 - $',ihex(maxi),') ? ');
          readln (stemp);
          val ('$'+stemp, anfang, i);

          if anfang > maxi then
            anfang := maxi;
          if anfang < 0 then
            i := 999;
        until i = 0;

        repeat
          gotoxy (x_anf,6);
          clreol;
          write ('FLLEN: Endadresse  Hi-Lo ($',ihex(anfang),
                 ' - $',ihex(maxi),') ? ');
          readln (stemp);
          val ('$'+stemp, endadr, i);
          if endadr > maxi then
            endadr := maxi;
          if endadr < anfang then
            i := 999;
        until i = 0;

        repeat
          gotoxy (x_anf,7);
          clreol;
          write ('FLLEN: Fllen mit             ($0 - $ff) ? ');
          readln (stemp);
          val ('$'+stemp, long_dumm, i);
          if long_dumm > 255 then
            i := 999;
        until i = 0;

        fuell_b := long_dumm;
        gotoxy (x_anf,7);
        clreol;
        gotoxy (x_anf,6);
        clreol;
        gotoxy (x_anf,5);
        clreol;
        if sicher ('FLLEN von $'
                  + ihex(anfang)
                  + ' bis $'
                  + ihex(endadr)
                  + ' mit $'
                  + hex(fuell_b)+' : ') then
          for i := anfang to endadr do
            eprom [i] := fuell_b;
      end;


    if ein = 'L' then
      file_lesen;


    if ein = 'S' then
      file_schreiben;


    if ein = 'G' then
      begin
        repeat
          gotoxy (x_anf,5);
          clreol;
          write ('GEHE ZU: Adresse Hi-Lo ($0 - $',ihex(maxi),') ? ');
          readln (stemp);
          val ('$'+stemp, anfang, i);                  { ANFANG ist dummy }

          if (anfang < 0) or (anfang > (maxi - $1f)) then
            begin
              pieps_twit;
              i := 999;
              gotoxy (x_anf,5);
              clreol;
              write ('ADRESSE falsch... (bitte nur $0 - $',
                     ihex(maxi - $1f),') !!! ');
              pieps_twit;
              delay (3000);
            end;
        until i = 0;

        x_m := anfang;
        gotoxy (x_anf,5);
        clreol;
      end;

    abbruch := false;
    if ein = 'Q' then
      begin
        if sicher('PROGRAMM BEENDEN ?? ') then
          abbruch := true;
      end;                                   { muss vor FUNKT-Tasten stehen }


    if ein = #0 then
      begin                                            { Sondertaste Anfang }
        ein := readkey;
        if ein = up then
          dec (y_pos);

        if ein = dn then
          inc (y_pos);

        if ein = rechts then
          inc (x_pos,2);

        if ein = links  then
          dec (x_pos,2);

        matrix_check;

        if ein = pgup then
          begin
            inc (x_m , $20);
            if x_m > maxi then
              x_m := maxi - $1f;
          end;

        if ein = pgdn then
          begin
            dec (x_m , $20);
            if x_m > maxi then
              x_m := 0;       { OBACHT !! Modulo 64k-Rechnung !! deswegen }
                              { Abfrage auf obere 32k                     }
          end;

        if ein = home then
          x_m := 0;

        if ein = ende then
          x_m := maxi - $1f;
      end;                                              { Sonder-Taste ENDE }

  until (ein in [pgup , pgdn , home , ende ,'F','L','A','S'])
        or abbruch;
end;                                                              { editier }

{.pa}
{---------------------------------------------------------------------------}
{ HAUPTPROGRAMM                                                             }
{---------------------------------------------------------------------------}
begin
  version := 'DC7TU 22.01.1993 ';
  normal;
  sirene;
  inverse;
  clrscr;
  titel2 ('LOGOMAT-RUFZEICHNER fr 2764 Eproms  ' + version);
  inverse;

  fillchar (eprom , sizeof (eprom) , $00);
  fname       := 'DC7TUxx.ROM';
  automat     := false;
  abbruch     := false;
  einfueg     := true;
  x_m         := 0;
  x_anf       := 8;
  y_anf       := 10;
  y_pos       := y_anf;

  repeat
    x_pos := x_anf;                       { X-Pos der Matrix auf dem Schirm }
    if automat then
      begin
        leer  := ' ';
        block := '';                                                 { 219 }
      end

    else

      begin
        gotoxy (x_anf-4,9);
        write ('Hex');
        gotoxy (x_anf-4,18);
        write ('Hi');
        gotoxy (x_anf-4,19);
        write ('Lo');
        gotoxy (x_anf-4,21);
        writf ('Quit');
        writf ('  Schreiben');
        writf ('  Lesen');
        writf ('  Fllen');
        writf ('  Automat');
        writf ('  Gehezu');
        writf ('      PgUp');
        writf (' PgDn');
        writf (' Home');
        writf (' End');
        gotoxy (x_anf-4,22);
{       write ('Quit  Schreiben  Lesen  Fllen  Automat  Gehezu'); }
        write ('Prog  File       File   vonbis           Adress');
        leer   := '.';
        block  := '';                                                  {223}
      end;

    for z := 0 to $1f do                    { immer 32 Byte am Stck zeigen }
      begin
        sc_byte := eprom [x_m + z];                  { Byte von EPROM holen }

        if not automat then
          begin
            gotoxy (x_pos , 9);
            write (hex(sc_byte));                   { Inhalt als HEX zeigen }
            gotoxy (x_pos , 18);
            write (hex (hi(x_m + z)));                   { EPROM-Adresse HI }
            gotoxy (x_pos , 19);
            write (hex (lo(x_m + z)));                   { EPROM-Adresse LO }

            zeig_byte;            { Byte als senkrechtes Punktmuster zeigen }
            inc (x_pos);          { nchste X-Position mit 1 Spalte Abstand }
            sc_byte := 0;
            leer := ' ';
            zeig_byte;
            inc (x_pos);
            leer := '.';
          end
        else
          begin
            zeig_byte;
            inc (x_pos);
            zeig_byte;
            inc (x_pos);
          end;

      end;

    if not automat then
      editier                          { die angezeigten 32 Bytes editieren }

    else

      begin
        delay (1000);                                             { AUTOMAT }
        inc (x_m,$20);
        if x_m > (bis - $1f) then
          x_m := 0;

        if keypressed then
          begin
            ein := readkey;
            if ein = ' ' then
              x_m := 0;
            automat := (ein <> esc);
          end;
      end;

    if abbruch
       and geaendert
       and not gespeichert then
      begin
        pieps_twit;
        gotoxy (x_anf , 23);
        inverse;
        write ('Abspeichern (J/N) ? ');
        ein := readkey;
        if upcase (ein) = 'J' then
          begin
            file_schreiben;
            abbruch := false;
          end;
        gotoxy (x_anf , 22);
        clreol;
      end;
  until abbruch;

  gotoxy (1,22);
  clreol;
  gotoxy (1,23);
  clreol;
  gotoxy (1,24);
  clreol;
  gotoxy (1,21);
  clreol;
  sirene;
  writeln ('  Tschuess ....');
  writeln ('  Dieses Programm wurde geschrieben fr DL3PD/DL6LG');
  writeln ('  und die UHF/SHF Gruppe Ludwigsburg/Backnang');
  writeln ('  von Peter Ochoisky');
  write   ('  Das war''s, Freunde...');
  delay (500);
  writeln;
  writeln;
end.
{---------------------------------------------------------------------------}

